// Copyright (c) 1999, A.H. van den Boogaard

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>
#ifdef __BORLANDC__
#pragma hdrstop
#endif
#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif

#include "dialogs.h"
#include "winfem2.h"
#include <wx/valgen.h>
#include <wx/statline.h>
#include "valnumbr.h"

DECLARE_APP( MyApp )

#define START_BUTTON  1
#define HALT_BUTTON   2
#define CANCEL_BUTTON 3

BEGIN_EVENT_TABLE(ControlDialog, wxDialog)
EVT_BUTTON (START_BUTTON,   ControlDialog::OnButton)
EVT_BUTTON (HALT_BUTTON,    ControlDialog::OnButton)
EVT_BUTTON (CANCEL_BUTTON,  ControlDialog::OnButton)
END_EVENT_TABLE()

//IMPLEMENT_DYNAMIC_CLASS(ControlDialog, wxDialog)

ControlDialog::ControlDialog( wxFrame *parent )
           :wxDialog( parent, -1, "FeaTure control dialog", wxPoint(100,100),
                      wxSize(350,250) )
{
  wxBoxSizer *topsizer = new wxBoxSizer( wxVERTICAL );
   wxBoxSizer *button_box = new wxBoxSizer( wxHORIZONTAL );
   int butw = 70;
   startbutt = new wxButton(this, START_BUTTON, "Start",
                            wxPoint(30,20),  wxSize(butw,-1) );
   stopbutt  = new wxButton(this, HALT_BUTTON,  "Halt",
                            wxPoint(110,20),  wxSize(butw,-1) );
   cancbutt  = new wxButton(this, CANCEL_BUTTON, "Cancel",
                            wxPoint(250,20), wxSize(butw,-1) );
   button_box->Add( startbutt, 0, wxALL, 7 );
   button_box->Add( stopbutt, 0, wxALL, 7 );
   button_box->AddSpacer( butw );
   button_box->Add( cancbutt, 0, wxALL, 7 );

   topsizer->Add( button_box, 0, wxALL, 7 );

#if wxUSE_GAUGE
   topsizer->Add( new wxStaticText(this, -1, "increment counter",
                              wxPoint(50,70), wxSize(-1,-1) ),
       0, wxLEFT | wxTOP | wxRIGHT , 14 );
   topsizer->AddSpacer( 3 );
      // Range is 100
      gauge = new wxGauge(this, -1, 100, wxPoint(50,90), wxSize(250,20),
                          wxGA_HORIZONTAL );
      gauge->SetShadowWidth(2);
   topsizer->Add( gauge, 0, wxEXPAND | wxLEFT | wxBOTTOM | wxRIGHT, 14 );
#endif

   topsizer->Add( new wxStaticText(this, -1, "convergence in force norm",
                           wxPoint(50,135), wxSize(-1,-1) ),
           0, wxLEFT | wxTOP | wxRIGHT, 14 );
   topsizer->AddSpacer( 3 );
   loadbar = new LoadBar( this, wxPoint(50,155), wxSize(250,50),
                          wxSIMPLE_BORDER );
   loadbar->SetLogarithmic( true );
   topsizer->Add( loadbar, 0, wxEXPAND | wxLEFT | wxBOTTOM | wxRIGHT, 14 );

   reset();

   SetSizer( topsizer );

   topsizer->SetSizeHints( this );
}

void ControlDialog::reset()
{
#if wxUSE_GAUGE
   gauge->SetValue(0);
#endif
   stop   = true;
   cancel = false;
   waiting = false;
   loadbar->Reset();
}

void ControlDialog::set_gauge_percent( int percent )
{
#if wxUSE_GAUGE
   gauge->SetValue( percent );
#endif
}

void ControlDialog::OnButton(wxCommandEvent& event)
{
   switch ( event.GetId() )
   {
      case CANCEL_BUTTON:
      {
         if ( wxMessageBox( "stop this analysis as soon as possible?",
                            "control", wxYES_NO )==wxYES )
         {
            if ( waiting && !cancel )
            {
               cancel = stop = true;
               get_main_frame()->ProcessCommand( MyFrame::FEA_STOP );
            }
            else
               cancel = stop = true;
         }
         return;
      }
      case HALT_BUTTON:
      {
         stop = (wxMessageBox( "stop after this increment?\n"
                               "(analysis can be continued later)",
                               "control", wxYES_NO )==wxYES);
         return;
      }
      case START_BUTTON:
      {
         cancel = false;

         if ( stop && waiting )
         {
            stop = false;
            get_main_frame()->ProcessCommand( MyFrame::FEA_RUN );
         }
         return;
      }
   }

   return;
}

BEGIN_EVENT_TABLE(DrawPropDialog, wxDialog)
EVT_BUTTON (wxID_OK,    DrawPropDialog::OnOK)
EVT_BUTTON (wxID_APPLY, DrawPropDialog::OnApply)
END_EVENT_TABLE()

DrawPropDialog::DrawPropDialog( MyFrame *parent, DrawProperties *dp,
                                const wxString& title,
                                const wxPoint& pos, const wxSize& size,
                                long style )
 :  wxDialog(parent, -1, title, pos, size, style ),
    drawprops(dp)
{
  mainFrame = parent;

  wxBoxSizer *topsizer = new wxBoxSizer( wxVERTICAL );

  wxGridSizer *checkgridsizer = new wxGridSizer( 2, 2, 5, 10 );
  checkgridsizer->Add(
        new wxCheckBox(this, 180, "Element numbers", wxDefaultPosition,
               wxDefaultSize, 0, wxGenericValidator(&(drawprops->elmnr)) ),
        0, wxALIGN_LEFT );
  checkgridsizer->Add(
        new wxCheckBox(this, 182, "Plot undeformed", wxDefaultPosition,
               wxDefaultSize, 0, wxGenericValidator(&(drawprops->undeformed)) ),
        0, wxALIGN_LEFT );
  checkgridsizer->Add(
        new wxCheckBox(this, 181, "Node numbers", wxDefaultPosition,
               wxDefaultSize, 0, wxGenericValidator(&(drawprops->nodenr)) ),
        0, wxALIGN_LEFT );
  checkgridsizer->Add(
        new wxCheckBox(this, 183, "Plot deformed", wxDefaultPosition,
               wxDefaultSize, 0, wxGenericValidator(&(drawprops->displaced)) ),
        0, wxALIGN_LEFT );

  topsizer->Add( checkgridsizer, 0, wxALL | wxALIGN_CENTER, 10 );

  wxGridSizer *textgridsizer = new wxGridSizer( 3, 2, 5, 5 );

  textgridsizer->Add(
        new wxStaticText(this, -1, "displacement scaling", wxDefaultPosition ),
                         0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL );
  textgridsizer->Add(
        new wxTextCtrl(this, 184, "", wxDefaultPosition, wxDefaultSize, 0,
                       DoubleTextValidator( &(drawprops->magnify) )),
        0, wxALIGN_LEFT );

  textgridsizer->Add(
        new wxStaticText(this, -1, "plot every n-th step: n=",
                         wxDefaultPosition ),
        0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL );
  textgridsizer->Add(
        new wxTextCtrl(this, 185, "", wxDefaultPosition, wxDefaultSize, 0,
                       IntTextValidator( &(drawprops->frequency) )),
        0, wxALIGN_LEFT );

  textgridsizer->Add(
        new wxStaticText(this, -1, "wait 'x' seconds per plot: x=",
                         wxDefaultPosition ),
        0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL );
  textgridsizer->Add(
        new wxTextCtrl(this, 186, "", wxDefaultPosition, wxDefaultSize, 0,
                       DoubleTextValidator( &(drawprops->slowdown) )),
        0, wxALIGN_LEFT );

  topsizer->Add( textgridsizer, 0, wxALL | wxALIGN_CENTER, 10 );

  topsizer->Add(
        new wxStaticLine( this, -1, wxDefaultPosition, wxDefaultSize,
                          wxHORIZONTAL),
        0, wxEXPAND | wxALL, 5 );
 
  wxBoxSizer *button_box = new wxBoxSizer( wxHORIZONTAL );

  wxButton *okBtn = new wxButton(this, wxID_OK, "OK", wxDefaultPosition );
  button_box->Add( okBtn, 0, wxALL, 7 );
  button_box->Add(
        new wxButton(this, wxID_APPLY, "Apply", wxDefaultPosition ),
        0, wxALL, 7 );
  button_box->Add(
        new wxButton(this, wxID_CANCEL, "Cancel", wxDefaultPosition ),
        0, wxALL, 7 );

  topsizer->Add( button_box, 0, wxCENTER | wxALL, 10 );

  SetAutoLayout( TRUE );
  SetSizer( topsizer );

  topsizer->Fit( this );
  topsizer->SetSizeHints( this );
 
  okBtn->SetFocus();
  okBtn->SetDefault();
} 

void DrawPropDialog::OnOK( wxCommandEvent& event )
{
   wxDialog::OnOK( event );
   mainFrame->canvas->Refresh();
}

void DrawPropDialog::OnApply( wxCommandEvent& event )
{
   wxDialog::OnApply( event );
   mainFrame->canvas->Refresh();
}
