// Copyright (c) 1999, A.H. van den Boogaard

#ifndef FTPARSER_H
#define FTPARSER_H

#include <cstdio>

#include "mydomain.h"
#include "mycontrol.h"
#include "elastic.h"
#include "plastic.h"

typedef FeaTure::IncrementalControl<MyDomain> MyControl;

class input_data
{
   private:
      typedef FeaTure::Node* Node_ptr;

      int    line_nr, open, nr_of_fields, key, new_key,
             error_in_file, error_in_line, current_field, interpreted;
      size_t field_start, field_end;
      std::FILE  *in_ptr;
      std::string filename;
      std::string separators;
      std::string input_line;
      std::string current_word;

      void read_line();
      void set_field( int field );
      void read_next_line();
      int  line_is_interpreted();
      int  get_key();
      int  is_key();
      int  is_element_name();
      void get_field( int field, int* item );
      void get_field( int field, double* item );
      void get_field( int field, std::string &item );
      void get_field_lower_case( int field, std::string &item );
      int  eof();
      void print_line();
      void print_key();
      int  open_file();
      int  read_nodes( MyDomain *model );
      int  read_title( MyDomain *model );
      int  read_elements( MyDomain *model );
      int  read_suppress( MyDomain *model );
      int  read_prescribed( MyDomain *model, FeaTure::LoadSet *load_set );
      int  read_nodal_loads( MyDomain *model, FeaTure::LoadSet *load_set );
      int  read_pressure_load( MyDomain *model, FeaTure::LoadSet *load_set );
      int  read_material( MyDomain *model );
      Elastic_type* read_elastic_material( int mat_nr );
      Elastic_type_xp* read_xp_elastic_material( int mat_nr );
      NonlinearElastic_type* read_nonlinearelastic_material( int mat_nr );
      VonMises_type* read_vonmises_material( int mat_nr );
      LinearThermal_type* read_thermal_material( int mat_nr );
      ThermoElastic_type* read_thermoelastic_material( int mat_nr );
      ThermoPlastic_type* read_thermoplastic_material( int mat_nr );
      FeaTure::Mass_type* read_mass_only_material( int mat_nr );
      int  read_geometry( MyDomain *model );
      int  read_nonlinear( FeaTure::Control<MyDomain> **control );
      int  read_linear( FeaTure::Control<MyDomain> **control );
      int  read_eigen( FeaTure::Control<MyDomain> **control );
      int  read_nonlinear_static( FeaTure::Control<MyDomain> **control );
      int  read_nonlinear_dynamic( FeaTure::Control<MyDomain> **control );
      int  read_nonlinear_explicit( FeaTure::Control<MyDomain> **control );
//      int  read_nonlinear( MyControl **control );
//      int  read_nonlinear_static( MyControl **control );
//      int  read_nonlinear_dynamic( MyControl **control );
//      int  read_nonlinear_explicit( MyControl **control );
      int  read_load_definition( MyDomain *model );
//      int  read_post( MyControl *control );
      int  read_post( FeaTure::Control<MyDomain> *control );
      bool interpret_incremental_iterative( const std::string &word,
                     FeaTure::IncrementalIterativeControl<MyDomain> &control );
      bool interpret_incremental( const std::string &word, MyControl &control );

      void create_structural_elm( int elm_key, MyDomain *model,
                                  FeaTure::Structural_type *mat,
                                  FeaTure::Geometry *geom,
                                  int elm_nr, Node_ptr *node_array );
      void create_thermal_elm( int elm_key, MyDomain *model,
                               FeaTure::Thermal_type *mat,
                               FeaTure::Geometry *geom,
                               int elm_nr, Node_ptr *node_array );
      void create_element( int elm_key, MyDomain *model,
                           FeaTure::Material_type *mat,
                           FeaTure::Geometry *geom,
                           int elm_nr, Node_ptr *node_array );
      void print_error_line();
      void print_field( int field );
      void get_direction( const std::string &input, FeaTure::Direction *dir );

      bool separator( char );
      char skip_blanks();
      bool is_dof_type( char test, FeaTure::Dof::Type *type );
      bool is_load_type( char test, FeaTure::Dof::Load *type );
      void skip_line();
      char read_word( std::string *input_line );

      Node_ptr* get_node_array( MyDomain *model, int rqd_nodes );

   public:
      input_data();
      input_data( std::string name );
      ~input_data();
      void set_file_name( const std::string& name ){ filename = name; }
      int  is_new_key();
      int  interpreted_all();
//      void interprete_line( MyDomain *model, MyControl **control );
      void interprete_line( MyDomain *model,
                            FeaTure::Control<MyDomain> **control );
      void set_interpreted();
      int  nr_of_errors();
};

#endif  // FTPARSER_H
