// Copyright (c) 1999, A.H. van den Boogaard

#ifndef PLASTIC_H
#define PLASTIC_H

#include "material.h"
#include "thermal.h"

/* advance declaration */
class VonMises_type;

class VonMises_Point:public FeaTure::Structural_Point
{
   protected:
      FeaTure::Vector strain_bar;

   private:
      VonMises_type *parameters;
      FeaTure::Vector e_elas0;
      FeaTure::Vector e_elas1;
      FeaTure::Vector prev_dstrain;

      double eq0, eq1, e_tilde, p0, p1;
      bool   plastic;
//      FeaTure::Vector strain_bar;

      FeaTure::Matrix get_xp_D_mat() const;

   public:
      VonMises_Point( VonMises_type *mat, FeaTure::StressType st );
      ~VonMises_Point();

      FeaTure::Matrix get_D_mat() const;
      FeaTure::Vector get_stress() const;
      void set_strain_increment( const FeaTure::Vector& strain_increment );
      void finish_increment();
      FeaTure::Vector get_xp_stress() const;
      double get_eps_plast_eq( bool &has_epq ) const;

      void GetRestartData( FeaTure::CollectArray &data ) const;
      void PutRestartData( const FeaTure::CollectArray &data );
};

/* advance declaration */
class ThermoPlastic_type;

class ThermoPlastic_Point:public VonMises_Point, public LinearThermal_Point
{
   protected:
      double thermal_strain_old, Delta_eps_th;

   public:
      ThermoPlastic_Point( ThermoPlastic_type *mat, FeaTure::StressType st );
      ~ThermoPlastic_Point();

      void set_strain_increment( const FeaTure::Vector& strain_increment );
      void set_potential_increment( double Tinc );
      void finish_increment();
      double get_expansion_coefficient() const;
      void ExpandEps( FeaTure::Vector& dstrain, FeaTure::Vector& ddstrain,
                      FeaTure::Vector& strain_bar );

      void GetRestartData( FeaTure::CollectArray &data ) const;
      void PutRestartData( const FeaTure::CollectArray &data );
};

/* Material type definitions */

class VonMises_type:public FeaTure::Structural_type
{
   private:
      double Young, Poisson, eps0, C, n;

   public:
      VonMises_type( int matnr );
      ~VonMises_type();

      int set_Young( double young );
      double get_Young() const;
      int set_Poisson( double poisson );
      double get_Poisson() const;
      int set_yield( double s0 );
      double get_eps0() const;
      int set_nadai_C( double c );
      double get_nadai_C() const;
      int set_nadai_n( double n );
      double get_nadai_n() const;

#ifdef NOVARIABLERETURN
      FeaTure::Material_Point* new_Material_Point( FeaTure::StressType st );
#else
      VonMises_Point* new_Material_Point( FeaTure::StressType st );
#endif
      bool NeedExpandStrain( FeaTure::StressType st ) const;
      bool symmetric() const { return true; }
};

class ThermoPlastic_type:public VonMises_type, public LinearThermal_type
{
   private:
      double alpha;

   public:
      ThermoPlastic_type( int matnr );
      ~ThermoPlastic_type();

#ifdef NOVARIABLERETURN
      FeaTure::Material_Point* new_Material_Point( FeaTure::StressType st );
#else
      ThermoPlastic_Point* new_Material_Point( FeaTure::StressType st );
#endif
      int set_alpha( double al );
      double get_alpha() const { return alpha; }
      bool symmetric() const { return false; }
};

#endif /* PLASTIC_H */
