// Copyright (c) 1999, 2005, A.H. van den Boogaard

#include <fstream>

#include "ftstring.h"

#include "read_mod.h"
#include "ftutils.h"

using std::cin;
using std::cout;
using std::cerr;
using std::endl;

// template definitions
// (putting control.cpp before using std::endl resulted in errors
//  on Borland C++ 5.5)
#include "domain.cpp"

void out_of_mem()
{
   cerr << "Sorry, out of memory\n";
   throw std::bad_alloc();
}

int main( int argc, char *argv[])
{
   char   ch;
   std::string filename, outfilename, postfilename;
   MyDomain *model = NULL;
   FeaTure::Control<MyDomain> *control = NULL;
   int    nr_errors;
   bool   batch;

   std::set_new_handler( out_of_mem );

   if ( argc==2 )
   {
      filename = argv[1];
      batch = true;
   }
   else batch = false;

   do
   {
     if ( !batch )
     {
        cout << endl << "give file name of the model: " << std::flush ;
        cin >> filename;
     }

     StopWatch timer;
     timer.start();

     std::string base_path;
     base_path = get_dirname(filename) + get_basename(filename);

     outfilename = base_path + ".out";
     std::ofstream outfile( outfilename.c_str() );

     std::ofstream postfile;

     nr_errors = read_model( filename, &model, &control );
     if ( nr_errors > 0 )
     {
        if ( nr_errors == 1 )
        {
           cerr << endl << "there is 1 error in input file "
                << filename << endl;
           outfile << endl << "there is 1 error in input file "
                   << filename << endl;
        }
        else
        {
           cerr << endl << "there are " << nr_errors
                << " errors in input file " << filename << endl;
           outfile << endl << "there are " << nr_errors
                   << " errors in input file " << filename << endl;
        }
        cerr << "correct the input file and try again" << endl;
        outfile << "correct the input file and try again" << endl;
     }
     else
     {
        model->print_title( outfile );

        if ( control->need_post_file() )
        {
           postfilename = base_path + ".flavia.res";
           postfile.open( postfilename.c_str() );
        }

        try
        {
           control->solve( *model, outfile, postfile );
        }
        catch ( const std::exception& err )
        {
           outfile << endl << "ERROR: " << err.what() << endl;
           cerr    << endl << "ERROR: " << err.what() << endl;
        }
     }
     
     delete model;
     delete control;
     model = NULL;
     control = NULL;

     cout << endl << "Time usage: " << timer.elapsed() << " seconds" << endl;

     if ( !batch )
     {
        cout << endl << "analyse another model? (y/n): ";
        cin >> ch;
     }
   }
   while ( !batch && ch=='y' );

   cout << endl << "finished" << endl;

   return 0;
}
