// Copyright (c) 1999, A.H. van den Boogaard

#ifndef TLELEMEN_H
#define TLELEMEN_H

#include "elements.h"
#include "tensor.h"

class TLNumIntStructElement:public NumIntStructElement
{
   private:
      void make_BL_mat_ps( const FeaTure::Matrix &dNdX, const Tensor2Gen &F0,
                           FeaTure::Matrix &BL ) const;
      void make_BL_mat_gen( const FeaTure::Matrix &dNdX, const Tensor2Gen &F0,
                            FeaTure::Matrix &BL ) const;
      void make_BL_mat_axi( const FeaTure::Matrix &dNdX, const Tensor2Gen &F0,
                            double r, const FeaTure::Matrix &N,
                            FeaTure::Matrix &BL ) const;

   protected:
      Tensor2Gen *defgrad0, *defgrad1;

      TLNumIntStructElement( int nr, Node_ptr *Nodes, int nr_nodes,
                             int struct_dofs, int nr_dofs,
                             FeaTure::Material_type *mat,
                             FeaTure:: Geometry *geom, FeaTure::StressType st,
                             int nip );
      ~TLNumIntStructElement();

      void WriteElementRestartData( std::ostream& restart ) const;
      void ReadElementRestartData( std::istream& restart );
      void iteration_update();
      void finish_increment();
      FeaTure::Matrix get_BL_mat( const FeaTure::Vector &natcor,
                                  const FeaTure::Matrix &coor_mat,
                                  const Tensor2Gen &F0, double* DetJ ) const;
      void get_TL_B_mats( const FeaTure::Vector &natcor,
                          const FeaTure::Matrix &coor_mat, const Tensor2Gen &F0,
                          FeaTure::Matrix &BL, FeaTure::Matrix &BNL,
                          double* DetJ ) const;

      FeaTure::Matrix get_dNdX( const FeaTure::Vector &natcor,
                                const FeaTure::Matrix &coor,
                                double *detj ) const;

   public:
      FeaTure::Matrix GetLocalStiff( FeaTure::Vector& correction ) const;
      FeaTure::Vector GetLocalLHS() const;
      void PrintXpStress( std::ostream& out ) const;
};

class TLNumInt2D:public TLNumIntStructElement
{
   protected:
      TLNumInt2D( int nr, Node_ptr *Nodes, int nr_nodes, int nr_dofs,
                  FeaTure::Material_type *mat, FeaTure::Geometry *geom,
                  FeaTure::StressType st, int nip );
      ~TLNumInt2D();

   public:
      void PrintStressHeader( std::ostream& out ) const;
      void PrintXpStressHeader( std::ostream& out ) const;
};

class TLQuad:public TLNumInt2D
{
   protected:
      TLQuad( int nr, Node_ptr *Nodes, int nr_nodes, int nr_dofs,
              FeaTure::Material_type *mat, FeaTure::Geometry *geom,
              FeaTure::StressType st, int nxi, int neta );
      ~TLQuad(){}
};

class TLQuad4:public TLQuad
{
   protected:
      TLQuad4( int nr, Node_ptr *Nodes, FeaTure::Material_type *mat,
               FeaTure::Geometry *geom, FeaTure::StressType st,
               int nxi, int neta );
      ~TLQuad4(){}

   public:
      static int reqd_nr_nodes() { return 4; }
};

class TLQuad8:public TLQuad
{
   protected:
      TLQuad8( int nr, Node_ptr *Nodes, FeaTure::Material_type *mat,
               FeaTure::Geometry *geom, FeaTure::StressType st,
               int nxi, int neta );
      ~TLQuad8(){}

   public:
      static int reqd_nr_nodes() { return 8; }
};

class TLQuad4psn:public TLQuad4
{
   public:
      TLQuad4psn( int nr, Node_ptr *Nodes, FeaTure::Material_type *mat,
                  FeaTure::Geometry *geom );
      ~TLQuad4psn(){}
};

class TLQuad8psn:public TLQuad8
{
   public:
      TLQuad8psn( int nr, Node_ptr *Nodes, FeaTure::Material_type *mat,
                  FeaTure::Geometry *geom );
      ~TLQuad8psn(){}
};

#endif // TLELEMEN_H
