// Copyright (c) 2000, A.H. van den Boogaard

/*
   File: winfem2.ccp
   The 'main' file, to start the graphical user interface for the demo
   finite element program 'FeaTureD'.
*/

#ifdef __GNUG__
   #pragma implementation "winfem2.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
   #pragma hdrstop
#endif

#ifndef WX_PRECOMP
   #include "wx/wx.h"
#endif

#if defined(__WXGTK__) || defined(__WXMOTIF__)
#include "featured.xpm"
#endif

#include "winfem2.h"
#include "printing.h"

#include <wx/timer.h>
#include <wx/metafile.h>
#include <wx/clipbrd.h>
#include <wx/fontdlg.h>
#include <wx/dcps.h>
#include <wx/accel.h>
#include <wx/printdlg.h>

#include "ftstring.h"
#include "ftutils.h"
#include "read_mod.h"

#include "dialogs.h"

#include <locale.h>

#include <fstream>

// temporary, untill include through other includes
#include "graphics.h"
#include "screenmap.h"

#include <algorithm>

using std::endl;

// template definitions
// (putting control.cpp before using std::endl resulted in errors
//  on Borland C++ 5.5)
#include "domain.cpp"

void start_run_disable();
void stop_run_enable();

// Declare frames
MyFrame   *frame = NULL;
wxMenu    *the_file_menu = NULL;

static MyDomain *model=NULL;
static FeaTure::Control<MyDomain> *control=NULL;
static DeviceMap ScreenMap;
static bool zoomselect=false;

static wxPrintData *printData = (wxPrintData*) NULL;
static std::ofstream outfile;
static std::ofstream postfile;
static bool active_file=false;
static bool running=false;
static bool can_continue=false;
static bool started=false;
static StopWatch timer;

// Must initialise these in OnInit, not statically
//wxFont    *itemFont;
wxFont    *textWindowFont;

// ----------------------------------------------------------------------------
// event tables
// ----------------------------------------------------------------------------
 
BEGIN_EVENT_TABLE(MyFrame, wxFrame)
 
    EVT_MENU(FEA_LOAD_FILE,   MyFrame::OnLoadFile)
    EVT_MENU_RANGE(wxID_FILE1, wxID_FILE9,
                              MyFrame::OnRecentFile)
    EVT_MENU(FEA_PRINT    ,   MyFrame::OnPrint)
    EVT_MENU(FEA_PRINT_PREVIEW,MyFrame::OnPrintPreview)
    EVT_MENU(FEA_PRINT_SETUP,MyFrame::OnPrintSetup)
//#if wxUSE_POSTSCRIPT
//    EVT_MENU(FEA_PRINT_EPS,   MyFrame::OnPrintEps)
//#endif
    EVT_MENU(FEA_QUIT     ,   MyFrame::OnQuit)
#ifdef __WXMSW__
#if wxUSE_METAFILE
    EVT_MENU(FEA_COPY_MF  ,   MyFrame::OnCopyMF)
#endif
    EVT_MENU(FEA_COPY_BMP ,   MyFrame::OnCopyBMP)
#endif
    EVT_MENU(FEA_CLEAR    ,   MyFrame::OnClear)
    EVT_MENU(FEA_DRAW_PROP,   MyFrame::OnDrawProp)
    EVT_MENU(FEA_GRAPH_FONT,  MyFrame::OnGraphFont)
    EVT_MENU(FEA_ELEM_FONT,   MyFrame::OnElemFont)
    EVT_MENU(FEA_NODE_FONT,   MyFrame::OnNodeFont)
    EVT_MENU(FEA_ZOOM_IN  ,   MyFrame::OnZoomIn)
    EVT_MENU(FEA_ZOOM_OUT ,   MyFrame::OnZoomOut)
    EVT_MENU(FEA_FIT      ,   MyFrame::OnFit)
    EVT_MENU(FEA_REFRESH  ,   MyFrame::OnRefresh)
    EVT_MENU(FEA_ABOUT    ,   MyFrame::OnAbout)

    EVT_MENU(FEA_RUN      ,   MyFrame::OnRun)
    EVT_MENU(FEA_CONTINUE ,   MyFrame::OnContinue)
    EVT_MENU(FEA_STOP     ,   MyFrame::OnStop)
 
//    EVT_SIZE  (MyFrame::OnSize)
    EVT_CLOSE (MyFrame::OnCloseWindow)

END_EVENT_TABLE()

BEGIN_EVENT_TABLE(MyCanvas, wxScrolledWindow)
 
    EVT_RIGHT_DOWN(MyCanvas::OnRightDown)
    EVT_LEFT_DOWN (MyCanvas::OnLeftDown)
    EVT_LEFT_UP   (MyCanvas::OnLeftUp)
    EVT_MOTION    (MyCanvas::OnMouseMove)
    EVT_PAINT     (MyCanvas::OnPaint)
    EVT_CHAR      (MyCanvas::OnChar)
 
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(GraphicsFrame, wxFrame)
 
    EVT_CLOSE (GraphicsFrame::OnCloseWindow)
 
END_EVENT_TABLE()

std::string out_file_name, post_file_name;

// Create a new application object: this macro will allow wxWindows to create
// the application object during program execution (it's better than using a
// static object for many reasons) and also declares the accessor function
// wxGetApp() which will return the reference of the right type (i.e. MyApp and
// not wxApp)
IMPLEMENT_APP( MyApp )

// The `main program' equivalent, creating the windows and returning the
// main frame
bool MyApp::OnInit(void)
{
  // always use decimal dots (some locales define decimal commas)
  setlocale(LC_NUMERIC,"C");

//  RegisterValidators();

 // Create a small font
//  itemFont = new wxFont(11, wxROMAN, wxNORMAL, wxNORMAL);
  textWindowFont = new wxFont(12, wxMODERN, wxNORMAL, wxNORMAL);

  printData = new wxPrintData;

//#if defined(__WXGTK__) || defined(__WXMOTIF__)
//    (*printData) = * wxThePrintSetupData;
//#endif

  // Create the main frame window
  frame = new MyFrame(NULL, "FeaTureD", wxPoint(1,1), wxSize(550,250) );
  SetTopWindow( frame );

  return true;
}

int MyApp::OnExit()
{
   return 1;
}

// Define my frame constructor
MyFrame::MyFrame(wxFrame *frame, const char *title, const wxPoint& pos,
                 const wxSize& size):
  wxFrame(frame, -1, title, pos, size )
{
  text_window = 0;

  // Give it a status bar
  CreateStatusBar(2);

  // Load icon and bitmap
  wxIcon main_icon(wxICON(featured));
  wxIcon graph_icon(wxICON(featured));

  SetIcon(main_icon);

  // Make a menubar
  wxMenu *file_menu = new wxMenu;
  the_file_menu = file_menu ;

  file_menu->Append(FEA_LOAD_FILE, "&Load file ...\tCtrl+L", "Load a data file");
  file_menu->AppendSeparator();
  file_menu->Append(FEA_PRINT,     "&Print graphics",   "Print graphics to printer");
  file_menu->Append(FEA_PRINT_SETUP,"Print &setup",     "Setup printer properties");
  file_menu->Append(FEA_PRINT_PREVIEW,"Print pre&view", "Preview graphics to print");
//#if wxUSE_POSTSCRIPT
//  file_menu->Append(FEA_PRINT_EPS, "Print to &EPS file", "Print encapsulated PostScript");
//#endif
  file_menu->AppendSeparator();
  file_menu->Append(FEA_QUIT     , "&Quit",              "Quit program");
  file_menu->AppendSeparator();
  RecentFiles.UseMenu(file_menu);

  wxMenu *edit_menu = new wxMenu;
#ifdef __WXMSW__
#if wxUSE_METAFILE
  edit_menu->Append(FEA_COPY_MF,   "Copy picture (&mf)", "Copy metafile to clipboard");
#endif
  edit_menu->Append(FEA_COPY_BMP,  "Copy picture (&bmp)\tCtrl+C","Copy bitmap to clipboard");
#endif
  edit_menu->Append(FEA_CLEAR    , "&Clear",             "Clear screen");

  wxMenu *view_menu = new wxMenu;
  view_menu->Append(FEA_DRAW_PROP, "&Draw prop...",      "Select drawing properties");
  view_menu->Append(FEA_GRAPH_FONT,"Select &font ...",   "Select font and size for graphics");
  view_menu->Append(FEA_ELEM_FONT, "Select &element font ...","Select font and size for element numbers");
  view_menu->Append(FEA_NODE_FONT, "Select &node font ...", "Select font and size for node numbers");
  view_menu->Append(FEA_ZOOM_IN  , "Zoom &In",           "Zoom in, graphics screen");
  view_menu->Append(FEA_ZOOM_OUT , "Zoom &Out",          "Zoom out, graphics screen");
  view_menu->AppendSeparator();
  view_menu->Append(FEA_FIT      , "&Fit",               "Fit model to screen");
  view_menu->Append(FEA_REFRESH  , "&Refresh",           "Refresh graphics screen");

  wxMenu *help_menu = new wxMenu;
  help_menu->Append(FEA_ABOUT    , "&About",             "About FeaTureD");

  menu_bar = new wxMenuBar;

  menu_bar->Append(file_menu,   "&File");
  menu_bar->Append(edit_menu,   "&Edit");
  menu_bar->Append(view_menu,   "&View");
  menu_bar->Append(help_menu,   "&Help");

  // Associate the menu bar with the frame
  SetMenuBar(menu_bar);

#if wxUSE_ACCEL
    // Accelerators
    wxAcceleratorEntry entries[2];
    entries[0].Set(wxACCEL_NORMAL, (int) 'q', FEA_QUIT );
    entries[1].Set(wxACCEL_CTRL  , (int) 'l', FEA_LOAD_FILE );
    wxAcceleratorTable accel(1, entries);
    SetAcceleratorTable(accel);
#endif

  ctl_dialog = new ControlDialog( this );

  drawpropDialog = new DrawPropDialog( this, &drawprops, "drawing properties" );

  // Make a text window
  text_window = new wxTextCtrl( this, -1, "", wxDefaultPosition, wxDefaultSize,
                              wxHSCROLL | wxTE_MULTILINE | wxTE_READONLY );

  text_window->SetFont(*textWindowFont);

  redirectcout = new wxStreamToTextRedirector( text_window, &std::cout );
  redirectcerr = new wxStreamToTextRedirector( text_window, &std::cerr );

  graphics_frame = new GraphicsFrame(this, wxString("FeaTureD graphics"),
                                     wxPoint(300,300), wxSize(400,300) );

  int width, height;
  graphics_frame->GetClientSize( &width, &height );

  graphics_frame->SetIcon(graph_icon);

  canvas = new MyCanvas( graphics_frame, wxPoint(0,0), wxSize(width,height), wxRETAINED );

  canvas->SetBackgroundColour(*wxWHITE);

  Show(true);
  graphics_frame->Show(true);
}

// Intercept menu commands

void MyFrame::OnRecentFile( wxCommandEvent& event )
{
/*
*  According to the manual GetSelection should be used, but this does not work.
*   const wxString& filename( RecentFiles.GetHistoryFile( event.GetSelection() ) );
*   wxString message;
*   message << "GetSelection():" << event.GetSelection();
*   wxMessageBox( message, filename, wxOK );
*/
   const wxString& filename( RecentFiles.GetHistoryFile( event.GetId() - wxID_FILE1) );
   
   std::string dirName = get_dirname( filename.c_str() );
   DirectoryName = dirName.c_str();

   if ( LoadFile( filename.c_str(), &model, &control ) )
      ctl_dialog->Show(TRUE);
}

void MyFrame::OnLoadFile( wxCommandEvent& /*event*/ )
{
   wxFileDialog fd( this, "Load model data", DirectoryName, "",
                          "Model files (*.dat)|*.dat| All files (*.*)|*.*", wxOPEN );

   if ( fd.ShowModal() == wxID_OK )
   {
      wxString fileName = fd.GetPath();
      DirectoryName = fd.GetDirectory();

      char* csp;
      csp = new char[ fileName.size()+1 ];
      strcpy( csp, fileName.c_str() );
#ifdef __WXMSW__
      wxUnix2DosFilename(csp);
#endif
      if ( LoadFile( csp, &model, &control ) )
      {
         ctl_dialog->Show(TRUE);
 
         RecentFiles.AddFileToHistory( csp );
      }
      delete[] csp;
   }
}

void MyFrame::OnRun( wxCommandEvent& /*event*/)
{
    if ( !started )
    {
       if ( !active_file || out_file_name.empty() ) return;

       start_run_disable();
       ctl_dialog->Show(TRUE);
       ctl_dialog->set_waiting(false);

//       if ( outfile.is_open() ) outfile.close();
       outfile.open( out_file_name.c_str() );
       model->print_title( outfile );

       if ( control->need_post_file() )
       {
          assert( !post_file_name.empty() );
//          if ( postfile.is_open() ) postfile.close();
          postfile.open( post_file_name.c_str() );
       }

       try
       {
          control->initialize( *model, outfile );
          started = true;

          ProcessCommand( FEA_CONTINUE );
          stop_run_enable();
       }
       catch ( const std::exception& e )
       {
          stop_run_enable();
          can_continue = false;
	  std::cerr    << endl << "ERROR: " << e.what() << endl;
          outfile << endl << "ERROR: " << e.what() << endl;
          ProcessCommand( FEA_STOP );
       }

       return;
    }
    else
       ProcessCommand( FEA_CONTINUE );
}

void MyFrame::OnContinue( wxCommandEvent& )
{
   if ( model == NULL || control == NULL || !started ) return;

   if ( !can_continue )
   {
      std::cout << "Can not continue stopped analysis" << endl;
      return;
   }

   FeaTure::IncrementalControl<MyDomain> *inccontrol;
   inccontrol = dynamic_cast< FeaTure::IncrementalControl<MyDomain>* >(control);

   start_run_disable();
   ctl_dialog->set_waiting( false );

   int inc_steps=0;
   wxStartTimer();
   timer.start();
   try
   {
      while( ctl_dialog->go_on() && control->continuate( *model, outfile, postfile, 1 ) )
      {
         if ( inccontrol )
         {
            int gaugePos = int(100*(inccontrol->get_fraction_executed( 
                               FeaTure::IncrementalControl<MyDomain>::STEPS )));
            ctl_dialog->set_gauge_percent( gaugePos );
         }

         wxYield(); //  Look for user input

         if ( drawprops.plot_displaced() &&
              drawprops.plot_steps() > 0 &&
              ++inc_steps >= drawprops.plot_steps() )
         {
            frame->ProcessCommand( FEA_REFRESH );
            inc_steps = 0;
         }

         // slow down the analysis while keeping it responsive
         long elapsed( wxGetElapsedTime( FALSE ) );
         long millitogo( (long)drawprops.slow_down()*1000 );
         while ( elapsed < millitogo && ctl_dialog->go_on() )
         {
            wxMilliSleep( std::min(long(250),millitogo-elapsed) );
            elapsed = wxGetElapsedTime( FALSE );
            millitogo = (long)drawprops.slow_down()*1000;
            wxYield();
         }

         wxStartTimer();
      }
   }
   catch ( const std::exception& e )
   {
      can_continue = false;
      std::cerr    << endl << "ERROR: " << e.what() << endl;
      outfile << endl << "ERROR: " << e.what() << endl;
   }
   timer.stop();

   if ( inccontrol )
   {
      int gaugePos = int(100*(inccontrol->get_fraction_executed(
                         FeaTure::IncrementalControl<MyDomain>::STEPS )));
      ctl_dialog->set_gauge_percent( gaugePos );
   }

   ProcessCommand( FEA_REFRESH );
   stop_run_enable();
   ctl_dialog->set_waiting( true );

   // fall through
   ProcessCommand( FEA_STOP );
}

void MyFrame::OnStop( wxCommandEvent& )
{
   if ( !active_file || running ) return;

   if ( ctl_dialog->go_on() || ctl_dialog->cancelled() )
   {
      active_file = false;
      ctl_dialog->Show(FALSE);
      outfile.close();
      postfile.close();
      if ( ctl_dialog->cancelled() )
         std::cout << "cancelled (analysis time: " << timer.elapsed() << " s)" << endl;
      else
         std::cout << "finished (analysis time: " << timer.elapsed() << " s)" << endl;
   }
   else
      std::cout << "stopped (analysis time: " << timer.elapsed() << " s)" << endl;
}

/*#if wxUSE_POSTSCRIPT
void MyFrame::OnPrintEps( wxCommandEvent& )
{
   wxPostScriptDC dc( "", true, this );
   if ( dc.Ok() )
   {
      if ( dc.StartDoc("FeaTureD graphics") )
      {
         dc.StartPage();
         Draw( dc );
         dc.EndPage();
         dc.EndDoc();
      }
   }
}
#endif*/

void MyFrame::OnPrint(wxCommandEvent& WXUNUSED(event))
{
    wxPrintDialogData printDialogData( *printData );
 
    wxPrinter printer( &printDialogData );
    FEAPrintout printout("FeaTureD printout");
    if (!printer.Print(this, &printout, TRUE))
    {
        if (wxPrinter::GetLastError() == wxPRINTER_ERROR)
            wxMessageBox("There was a problem printing.\nPerhaps your current printer is not set correctly?", "Printing", wxOK);
        else
            wxMessageBox("You canceled printing", "Printing", wxOK);
    }
    else
    {
        (*printData) = printer.GetPrintDialogData().GetPrintData();
    }
}

void MyFrame::OnPrintPreview(wxCommandEvent& WXUNUSED(event))
{
    // Pass two printout objects: for preview, and possible printing.
    wxPrintDialogData printDialogData(*printData);
    wxPrintPreview *preview = new wxPrintPreview( new FEAPrintout, new FEAPrintout,
                                                  &printDialogData );
    if (!preview->Ok())
    {
        delete preview;
        wxMessageBox("There was a problem previewing.\nPerhaps your current printer is not set correctly?", "Previewing", wxOK);
        return;
    }
 
    wxPreviewFrame *frame = new wxPreviewFrame(preview, this,
                 "FeaTureD Print Preview", wxPoint(100, 100), wxSize(600, 650));
    frame->Centre(wxBOTH);
    frame->Initialize();
    frame->Show(TRUE);
}
 
void MyFrame::OnPrintSetup(wxCommandEvent& WXUNUSED(event))
{
    wxPrintDialogData printDialogData(* printData);
    wxPrintDialog printerDialog(this, & printDialogData);
 
    printerDialog.GetPrintDialogData().SetSetupDialog(TRUE);
    printerDialog.ShowModal();
 
    (*printData) = printerDialog.GetPrintDialogData().GetPrintData();
}

#ifdef __WXMSW__
#if wxUSE_METAFILE
void MyFrame::OnCopyMF( wxCommandEvent& )
{
   wxMetafileDC dc;
   if ( dc.Ok() )
   {
      Draw( dc );
      wxMetafile *mf = dc.Close();
      if ( mf )
      {
         //bool succes =
         mf->SetClipboard( (int)(dc.MaxX()+10), (int)(dc.MaxY()+10) );
         delete mf;
      }
   }
}
#endif // wxUSE_METAFILE

void MyFrame::OnCopyBMP( wxCommandEvent& )
{
   int width, height;
   frame->canvas->GetClientSize( &width, &height );
   wxMemoryDC dc;
   wxBitmap *bitmap = new wxBitmap(width,height);
   if ( !bitmap->Ok() )
   {
      wxMessageBox( "Could not create bitmap.", "FeaTureD Graphics", wxOK );
      return;
   }

   dc.SelectObject(*bitmap);
   dc.Clear();
   Draw(dc);
   dc.SelectObject(wxNullBitmap);

   if ( wxTheClipboard->Open() )
   {
      if ( !wxTheClipboard->SetData( new wxBitmapDataObject( *bitmap ) ) )
         wxMessageBox( "Could not copy bitmap to clipboard.", "FeaTureD Graphics", wxOK );
      wxTheClipboard->Close();
   }
   else
      wxMessageBox( "Could not open clipboard.", "FeaTureD Graphics", wxOK );

   delete bitmap;
}
#endif // __WXMSW__

void MyFrame::OnQuit( wxCommandEvent& )
{
   Close(false);
}

void MyFrame::OnClear( wxCommandEvent& )
{
   text_window->Clear();
   SetStatusText( "", 1 );
   canvas->ClearBackground();
}

void MyFrame::OnDrawProp( wxCommandEvent& )
{

  drawpropDialog->Centre(wxBOTH);
  drawpropDialog->Show(TRUE);
}

void MyFrame::OnGraphFont( wxCommandEvent& )
{
   wxFontData data;

   data.SetInitialFont( drawprops.graphicsFont );
   data.SetColour( drawprops.graphicsFontColor );

   wxFontDialog dialog( this, &data );
   if ( dialog.ShowModal() == wxID_OK )
   {
      wxFontData retData = dialog.GetFontData();
      drawprops.graphicsFont      = retData.GetChosenFont();
      drawprops.graphicsFontColor = retData.GetColour();
      frame->ProcessCommand( FEA_REFRESH );
   }
}

void MyFrame::OnElemFont( wxCommandEvent& )
{
   wxFontData data;

   data.SetInitialFont( drawprops.elementFont );
   data.SetColour( drawprops.elementFontColor );
   wxFontDialog dialog( this, &data );

   if ( dialog.ShowModal() == wxID_OK )
   {
      wxFontData retData = dialog.GetFontData();
      drawprops.elementFont      = retData.GetChosenFont();
      drawprops.elementFontColor = retData.GetColour();
      frame->ProcessCommand( FEA_REFRESH );
   }
}

void MyFrame::OnNodeFont( wxCommandEvent& )
{
   wxFontData data;

   data.SetInitialFont( drawprops.nodeFont );
   data.SetColour( drawprops.nodeFontColor );

   wxFontDialog dialog( this, &data );
   if ( dialog.ShowModal() == wxID_OK )
   {
      wxFontData retData = dialog.GetFontData();
      drawprops.nodeFont      = retData.GetChosenFont();
      drawprops.nodeFontColor = retData.GetColour();
      frame->ProcessCommand( FEA_REFRESH );
   }
}

void MyFrame::OnZoomIn( wxCommandEvent& )
{
   SetStatusText("click corner and drag to opposite", 0);
   zoomselect=true;
   canvas->SetCursor( wxCURSOR_CROSS );
}

void MyFrame::OnZoomOut( wxCommandEvent& )
{
   int width, height;
   float lwidth, lheight, xmid, ymid;

   wxClientDC dc( canvas );
   canvas->GetClientSize( &width, &height );
   lwidth  = dc.DeviceToLogicalXRel( width );
   lheight = dc.DeviceToLogicalYRel( height );
   xmid = ScreenMap.toX(lwidth/2);
   ymid = ScreenMap.toY(lheight/2);

   /* shift must have size: 1-zoom */
   ScreenMap.shiftLeft( ScreenMap.toScreen(-xmid*0.3 ));
   ScreenMap.shiftUp( ScreenMap.toScreen(ymid*0.3 ));
   ScreenMap.zoom( 0.7 );
   ProcessCommand( FEA_REFRESH );
}

void MyFrame::ShiftGraphics( float left, float up )
{
   int width, height;
   float lwidth, lheight;

   wxClientDC dc( canvas );

   canvas->GetClientSize( &width, &height );
   lwidth  = dc.DeviceToLogicalXRel( width );
   lheight = dc.DeviceToLogicalYRel( height );
   ScreenMap.shiftLeft( lwidth*left*0.2 );
   ScreenMap.shiftUp( lheight*up*0.2 );
   ProcessCommand( FEA_REFRESH );
}

void MyFrame::OnFit( wxCommandEvent& )
{
   MapFullScreen();  // fall through
   ProcessCommand( FEA_REFRESH );
}

void MyFrame::OnRefresh( wxCommandEvent& /*event*/ )
{
   canvas->Refresh();
}

void MyFrame::OnAbout( wxCommandEvent& )
{
   wxMessageBox( "FeaTureD 0.5\nAuthor: Ton van den Boogaard (c) 1999-2006",
                 "About FeaTureD", wxOK | wxCENTRE | wxICON_INFORMATION );
}

void MyFrame::Draw(wxDC& dc )
{
  dc.SetBackground(*wxWHITE_BRUSH);
  dc.Clear();

  dc.SetBackgroundMode( wxTRANSPARENT );
  dc.SetPen(*wxBLACK_PEN);

  if ( model != NULL )
  {
     dc.SetBrush(*wxCYAN_BRUSH);

     if ( drawprops.plot_displaced() )
     {
        model->draw_displaced( dc, ScreenMap, drawprops );

        dc.SetFont( drawprops.graphicsFont );
        dc.SetTextForeground( drawprops.graphicsFontColor );

        char s[30];
        sprintf( s, "deformation scaling: %8.3e", drawprops.displa_scale() );
        dc.DrawText( s, 1, 1 );

        if ( drawprops.plot_undeformed() )
        {
           wxPen *dash_pen = new wxPen("RED", 1, wxSHORT_DASH);
           dc.SetPen( *dash_pen );
           dc.SetBrush(*wxTRANSPARENT_BRUSH);
           model->draw_elements( dc, ScreenMap, false );
           dc.SetPen( wxNullPen );
           delete dash_pen;
        }
     }
     else if ( drawprops.plot_undeformed() )
     {
        dc.SetFont( drawprops.elementFont );
        dc.SetTextForeground( drawprops.elementFontColor );
        model->draw_elements( dc, ScreenMap, drawprops.plot_element_nrs() );

        dc.SetBrush(*wxBLACK_BRUSH);
        dc.SetFont( drawprops.nodeFont );
        dc.SetTextForeground( drawprops.nodeFontColor );
        model->draw_nodes( dc, ScreenMap, drawprops.plot_node_nrs() );
     }
  }

  dc.SetFont( wxNullFont );
}

// Define the behaviour for the frame closing

void MyFrame::OnCloseWindow(wxCloseEvent& event)
{
   if ( running && event.CanVeto() )
   {
      event.Veto(true);
      wxMessageBox( "Stop the running analysis first!" );
   }
   else
   {
      ctl_dialog->Destroy();
      drawpropDialog->Destroy();

      delete redirectcout;
      delete redirectcerr;

//      graphics_frame->Show(FALSE);
//      graphics_frame->Close(TRUE); // smooth way to delete graphics_frame;
      Show(false);
      Destroy();
   }
}

// Define a constructor for my canvas
MyCanvas::MyCanvas( wxFrame *parent, const wxPoint& pos, const wxSize& size, long style):
               wxScrolledWindow(parent, -1, pos, size, style)
{
   parent_frame = parent;
}

MyCanvas::~MyCanvas(void)
{
}

// Define the repainting behaviour
void MyCanvas::OnPaint( wxPaintEvent& )
{
  wxPaintDC dc( this );
  PrepareDC(dc);
  frame->Draw( dc );
}

void MyCanvas::OnRightDown(wxMouseEvent& event)
{
  wxMenu popupMenu;
  popupMenu.Append(MyFrame::FEA_DRAW_PROP, "&Draw prop...",      "Select drawing properties");
  popupMenu.Append(MyFrame::FEA_GRAPH_FONT,"Select &font ...",   "Select font and size");
  popupMenu.Append(MyFrame::FEA_ELEM_FONT, "Select &element font ...","Select font and size for element numbers");
  popupMenu.Append(MyFrame::FEA_NODE_FONT, "Select &node font ...", "Select font and size for node numbers");
  popupMenu.Append(MyFrame::FEA_ZOOM_IN  , "Zoom &In",           "Zoom in, graphics screen");
  popupMenu.Append(MyFrame::FEA_ZOOM_OUT , "Zoom &Out",          "Zoom out, graphics screen");
  popupMenu.AppendSeparator();
  popupMenu.Append(MyFrame::FEA_FIT      , "&Fit",               "Fit model to screen");
  popupMenu.Append(MyFrame::FEA_REFRESH  , "&Refresh",           "Refresh graphics screen");
  popupMenu.AppendSeparator();
#if wxUSE_POSTSCRIPT
  popupMenu.Append(MyFrame::FEA_PRINT_EPS, "Print to &EPS file", "Print encapsulated PostScript");
#endif
#ifdef __WXMSW__
  popupMenu.Append(MyFrame::FEA_COPY_BMP,  "Copy picture (&bmp)","Copy bitmap to clipboard");
#endif

  PopupMenu( &popupMenu, event.GetX(), event.GetY() );
}



static long zoom_x0=-1, zoom_y0=-1, zoom_x1, zoom_y1;
static bool rectDrawn=false;

void MyCanvas::OnLeftDown(wxMouseEvent& event)
{
   long x, y;
   event.GetPosition(&x, &y);
   wxClientDC dc( this );

   if ( zoomselect )
   {
      if ( rectDrawn )  // probably left screen before LeftUp
      {
         dc.SetPen(*wxBLACK_PEN);
         dc.SetBrush(*wxTRANSPARENT_BRUSH);
         dc.SetLogicalFunction(wxINVERT);
         dc.DrawRectangle(zoom_x0,zoom_y0,zoom_x1-zoom_x0,zoom_y1-zoom_y0);
         rectDrawn = false;
      }
      zoom_x0 = x;
      zoom_y0 = y;
      frame->SetStatusText("drag to oposite corner", 0);
   }
}

void MyCanvas::OnLeftUp(wxMouseEvent& /*event*/)
{
   if ( zoomselect )
   {
      wxClientDC dc( this );

      int width, height;
      float lwidth, lheight;
      float xmin, xmax, ymin, ymax, sety, setx, fac, scale;

      xmin = ScreenMap.toX( zoom_x0 );
      xmax = ScreenMap.toX( zoom_x1 );
      ymin = ScreenMap.toY( zoom_y1 );
      ymax = ScreenMap.toY( zoom_y0 );

      GetClientSize( &width, &height );
      lwidth  = dc.DeviceToLogicalXRel( width );
      lheight = dc.DeviceToLogicalYRel( height );

      if ( width != 0 && height != 0 )
      {
         fac = std::max( std::fabs(xmax-xmin)/lwidth, std::fabs(ymax-ymin)/lheight );
         if ( fac > 1.0e-9 )
         {
            scale = 1/fac;
            sety = (lheight+(ymin+ymax)*scale)/2  ;
            setx = ((xmin+xmax)*scale-lwidth)/2   ;
            ScreenMap.setOrigin( setx, sety );
            ScreenMap.setScale( scale );
         }
      }

      rectDrawn = false;
      zoomselect = false;
      zoomselect = false;
      SetCursor( wxNullCursor );

      frame->SetStatusText("", 0);
      ClearBackground();
      frame->ProcessCommand( MyFrame::FEA_REFRESH );
   }
}

void MyCanvas::OnMouseMove(wxMouseEvent& event)
{
   long x, y;
   event.GetPosition(&x, &y);
   wxClientDC dc( this );

   if ( zoomselect )
   {
      if ( event.Dragging() && event.LeftIsDown() )
      {
         dc.SetPen(*wxBLACK_PEN);
         dc.SetBrush(*wxTRANSPARENT_BRUSH);
         dc.SetLogicalFunction(wxINVERT);
         if ( rectDrawn ) dc.DrawRectangle(zoom_x0,zoom_y0,zoom_x1-zoom_x0,zoom_y1-zoom_y0);
         zoom_x1=x;
         zoom_y1=y;
         dc.DrawRectangle(zoom_x0,zoom_y0,zoom_x1-zoom_x0,zoom_y1-zoom_y0);
         rectDrawn = true;
         dc.SetLogicalFunction(wxCOPY);
      }
   }
}

// Intercept character input
void MyCanvas::OnChar(wxKeyEvent& event)
{
  switch ( event.GetKeyCode() )
  {
    case WXK_LEFT:   frame->ShiftGraphics(  1.0,  0.0 );
                     break;
    case WXK_RIGHT:  frame->ShiftGraphics( -1.0,  0.0 );
                     break;
    case WXK_UP:     frame->ShiftGraphics(  0.0,  1.0 );
                     break;
    case WXK_DOWN:   frame->ShiftGraphics(  0.0, -1.0 );
                     break;
    case WXK_HOME:   frame->ProcessCommand( MyFrame::FEA_FIT );
                     break;

    case '-':        frame->ProcessCommand( MyFrame::FEA_ZOOM_OUT );
                     break;
    case 'q':        frame->ProcessCommand( MyFrame::FEA_QUIT );
                     break;
    default:         // Process the default behaviour
                     event.Skip();
  }
}

void MyFrame::MapFullScreen()
{
   if ( model == NULL ) return;

   double xmin, ymin, zmin, xmax, ymax, zmax, xextend, yextend;
   int    width, height;
   float  lwidth, lheight, scale, dscale;

   if ( drawprops.plot_displaced() )
      dscale = drawprops.displa_scale();
   else
      dscale = 0.0;

   model->get_extremes( xmin, ymin, zmin, xmax, ymax, zmax, dscale );
   xextend = (xmax - xmin)*1.2+0.01;
   yextend = (ymax - ymin)*1.2+0.01;

   wxClientDC dc( frame->canvas );
   frame->canvas->GetClientSize( &width, &height );
   lwidth  = dc.DeviceToLogicalXRel( width )+1;
   lheight = dc.DeviceToLogicalYRel( height )+1;
   scale = std::min( lwidth/xextend, lheight/yextend );
   float sety, setx;
   sety = (lheight+(ymin+ymax)*scale)/2  ;
   setx = ((xmin+xmax)*scale-lwidth)/2   ;
   ScreenMap.setOrigin( setx, sety );
   ScreenMap.setScale( scale );
}

bool MyFrame::LoadFile( std::string s, MyDomain **model,
                        FeaTure::Control<MyDomain> **control )
{
   int nr_errors;

   delete *model;   *model = NULL;
   delete *control; *control = NULL;

   std::cout << "Starting " << s << endl;
   nr_errors = read_model( s, model, control );
   if ( nr_errors == 0 )
   {
      std::string base_path;
      base_path = get_dirname(s) + get_basename(s);

      out_file_name = base_path + ".out";
      post_file_name = base_path + ".flavia.res";

      frame->SetStatusText( wxFileNameFromPath(const_cast<char*>(s.c_str())), 1 );
      MapFullScreen();
      frame->ProcessCommand( MyFrame::FEA_REFRESH );

      active_file = true;
      can_continue = true;
      started = false;
      timer.reset();

      ctl_dialog->reset();
      ctl_dialog->set_waiting( true );

      return true;
   }
   else
   {
      std::cerr << "there are errors in the input file" << endl;
      return false;
   }
}


void start_run_disable()
{
   running = true;
   the_file_menu->Enable(MyFrame::FEA_LOAD_FILE, false );
   the_file_menu->Enable(wxID_FILE1, false );
   the_file_menu->Enable(wxID_FILE2, false );
   the_file_menu->Enable(wxID_FILE3, false );
   the_file_menu->Enable(wxID_FILE4, false );
   the_file_menu->Enable(wxID_FILE5, false );
   the_file_menu->Enable(wxID_FILE6, false );
   the_file_menu->Enable(wxID_FILE7, false );
   the_file_menu->Enable(wxID_FILE8, false );
   the_file_menu->Enable(wxID_FILE9, false );
}

void stop_run_enable()
{
   running = false;
   the_file_menu->Enable(MyFrame::FEA_LOAD_FILE, true );
   the_file_menu->Enable(wxID_FILE1, true );
   the_file_menu->Enable(wxID_FILE2, true );
   the_file_menu->Enable(wxID_FILE3, true );
   the_file_menu->Enable(wxID_FILE4, true );
   the_file_menu->Enable(wxID_FILE5, true );
   the_file_menu->Enable(wxID_FILE6, true );
   the_file_menu->Enable(wxID_FILE7, true );
   the_file_menu->Enable(wxID_FILE8, true );
   the_file_menu->Enable(wxID_FILE9, true );
}

MyFrame* get_main_frame(){ return frame; }

GraphicsFrame::GraphicsFrame(wxWindow *parent, const wxString& title, const wxPoint& pos, const wxSize& size)
          :wxFrame( parent, -1, title, pos, size,
           wxCAPTION | wxMINIMIZE_BOX | wxMAXIMIZE_BOX | wxTHICK_FRAME ){}

void GraphicsFrame::OnCloseWindow(wxCloseEvent& event)
{
   if ( event.CanVeto() )
   {
      event.Veto(true);
      wxMessageBox( "The graphics window can not be deleted separately",
                    "Graphics frame deletion message" );
      return;
   }
   else
   {
      Show(false);
      Destroy();
   }
}

void show_criterion( float crit )
{
   get_main_frame()->ctl_dialog->set_criterion( crit );
}

void show_convergence( float unb, bool highlight )
{
   get_main_frame()->ctl_dialog->push_convergence( unb, highlight );
}

