// Copyright (c) 2000, A.H. van den Boogaard

/* File:    winfem.h */
#ifndef WINFEM2_H
#define WINFEM2_H

#ifdef __GNUG__
   #pragma interface "winfem2.h"
#endif

#include <wx/docview.h>
#include "mycontrol.h"
#include "dialogs.h"

// Define a new application
class MyApp: public wxApp
{
  public:
    MyApp(void){}
    bool OnInit(void);
    int OnExit();
};

class GraphicsFrame: public wxFrame
{
  DECLARE_EVENT_TABLE()

  public:
    GraphicsFrame(wxWindow *parent, const wxString& title, const wxPoint& pos, const wxSize& size);
    void OnCloseWindow(wxCloseEvent& event);
};

// Define a new canvas which can receive some events
class MyCanvas: public wxScrolledWindow
{
  private:
    wxFrame *parent_frame;

  DECLARE_EVENT_TABLE()

  public:
    MyCanvas(wxFrame *parent, const wxPoint& pos, const wxSize& size, long style = wxRETAINED);
    ~MyCanvas(void) ;
    void OnPaint(wxPaintEvent& event);
    void OnChar(wxKeyEvent& event);
    void OnRightDown(wxMouseEvent& event);
    void OnLeftDown(wxMouseEvent& event);
    void OnLeftUp(wxMouseEvent& event);
    void OnMouseMove(wxMouseEvent& event);
};

// Define a new main frame

class MyFrame: public wxFrame
{
  public:
    enum MenuCommands{
       FEA_LOAD_FILE = 10,
       FEA_PRINT    ,
       FEA_PRINT_SETUP,
       FEA_PRINT_PREVIEW,
       FEA_PRINT_EPS,
       FEA_QUIT     ,
       FEA_COPY_MF  ,
       FEA_SAVE_MF  ,
       FEA_COPY_BMP ,
       FEA_CLEAR    ,
       FEA_DRAW_PROP,
       FEA_GRAPH_FONT,
       FEA_ELEM_FONT,
       FEA_NODE_FONT,
       FEA_ZOOM_IN  ,
       FEA_ZOOM_OUT ,
       FEA_FIT      ,
       FEA_REFRESH  ,
       FEA_ABOUT    ,

       FEA_RUN      ,
       FEA_CONTINUE ,
       FEA_STOP     ,

       FEA_NUMBERS  ,
       FEA_SHIFT_LEFT,
       FEA_SHIFT_RIGHT,
       FEA_SHIFT_UP,
       FEA_SHIFT_DOWN
    };

    DECLARE_EVENT_TABLE()

  public:
//    wxPanel       *panel;
    //LogScreen     *text_window;
    wxTextCtrl    *text_window;
    MyCanvas      *canvas;
    GraphicsFrame *graphics_frame;
    ControlDialog *ctl_dialog;

    MyFrame(wxFrame *frame, const char *title, const wxPoint& pos, const wxSize& size );
//    void OnSize(wxSizeEvent& event);
    void OnCloseWindow(wxCloseEvent& event);
    void OnMenuCommand(int id);
    void OnActivate(bool) {}

    void OnLoadFile( wxCommandEvent& event );
    void OnRecentFile( wxCommandEvent& event );
    void OnPrint( wxCommandEvent& event );
    void OnPrintPreview( wxCommandEvent& event );
    void OnPrintSetup( wxCommandEvent& event );
#if wxUSE_POSTSCRIPT
    void OnPrintEps( wxCommandEvent& event );
#endif
    void OnQuit( wxCommandEvent& event );
#ifdef __WXMSW__
#if wxUSE_METAFILE
    void OnCopyMF( wxCommandEvent& event );
#endif
    void OnCopyBMP( wxCommandEvent& event );
#endif
    void OnClear( wxCommandEvent& event );
    void OnDrawProp( wxCommandEvent& event );
    void OnGraphFont( wxCommandEvent& event );
    void OnElemFont( wxCommandEvent& event );
    void OnNodeFont( wxCommandEvent& event );
    void OnZoomIn( wxCommandEvent& event );
    void OnZoomOut( wxCommandEvent& event );
    void OnFit( wxCommandEvent& event );
    void OnRefresh( wxCommandEvent& event );
    void OnAbout( wxCommandEvent& event );

    void OnRun( wxCommandEvent& event );
    void OnContinue( wxCommandEvent& event );
    void OnStop( wxCommandEvent& event );

    void Draw(wxDC& dc );
    void ShiftGraphics( float left, float up );

  private:
    wxStreamToTextRedirector *redirectcout;
    wxStreamToTextRedirector *redirectcerr;
    wxFileHistory RecentFiles;
    wxString DirectoryName;
    DrawProperties drawprops;
    DrawPropDialog *drawpropDialog;
    wxMenuBar *menu_bar;

    bool LoadFile( std::string s, MyDomain **model,
                   FeaTure::Control<MyDomain> **control );
    void MapFullScreen();
};

MyFrame* get_main_frame();

#endif // WINFEM2_H
