// Copyright (c) 1999, A.H. van den Boogaard

#ifndef ELEMBASE_H
#define ELEMBASE_H

#include "ftmatrix.h"
#include "sysmat.h"
#include "sysvec.h"
#include "nodes.h"
#include "transmat.h"
#include "postproc.h"

namespace FeaTure
{

class ElementBase
{
   private:
      static double time_increment;
      static PostProc::PostItem post_item;

   protected:
      typedef Node* Node_ptr;

      class LocalDof
      {
         public:
            int       loc_node_nr;
            Direction dir;
            Dof::Type phys_type;
      };
      int      nr_of_element_nodes, elm_nr;
      int      nr_of_system_dofs, nr_of_local_dofs;
      Dof_ptr  *dofs;
      Node_ptr *Nodes;
      TransformationMatrix   transmat;

      ElementBase();
      ElementBase( int nr, Node_ptr *Nodes, int nr_of_element_nodes, int ndofs );

      Vector get_local_total_vector() const;
      Vector get_local_increment_vector() const;
      Vector GetLocalStressCorrection() const;
      static double GetTimeIncrement(){ return time_increment; }
      static PostProc::PostItem GetPostItem(){ return post_item; }

      virtual bool NeedExpandStrain() const {return false;}
      virtual Matrix GetLocalStiff( Vector& correction ) const=0;
      virtual Matrix GetLocalMass( bool lumped ) const;
      virtual Vector GetLocalLHS() const=0;
      virtual LocalDof* GetLocalDofs() const=0;
      virtual Vector GetLocalPresVec( int surface, double magnitude ) const;
      virtual void WriteElementRestartData( std::ostream &restart ) const=0;
      virtual void ReadElementRestartData( std::istream &restart ) =0;

   public:
      virtual ~ElementBase();
      virtual void iteration_update()=0;
      virtual void finish_increment()=0;
      virtual void AddNodalPostContribution() const { return; }

      static void SetTimeIncrement( double dtime ){ time_increment = dtime; }
      static void SetPostItem( PostProc::PostItem post ){ post_item = post; }
      virtual bool symmetric() const=0;
      int get_nr() const { return elm_nr; }

      void add_dofs_to_nodes() const;
      void CreateGlobalDofs();

      Vector GetGlobalLHS() const;
      void PutLHS_in_Global( SystemVector& lhs ) const;

      void InitStiff_in_Global( SystemMat *System ) const;
      void PutStiff_and_RHS_in_Global( SystemMat *System, Vector& rhs ) const;
      void PutStiff_in_Global( SystemMat *System, Vector& rhs ) const;

      void InitMass_in_Global( SystemMat *System, bool lumped ) const;
      void PutMass_in_Global( SystemMat *System, bool lumped ) const;

      void AddPres2Load( int surface, double magnitude,
                         SystemVector& load ) const;

      void WriteRestartData( std::ostream &restart ) const;
      void ReadRestartData( std::istream &restart );
};

} // namespace FeaTure

#endif // ELEMBASE_H
