// Copyright (c) 1999, A.H. van den Boogaard

#ifndef NODES_H
#define NODES_H

#include "cartvect.h"
#include "sysvec.h"
#include "ftmatrix.h"
#include "dofs.h"

namespace FeaTure
{

class Node
{
  private:
    int      node_nr;
    Location coordinates;
    static double eps_direct;

  protected:
    int      nr_nodal_dofs;
    Dof_ptr  *dofs;

  public:
    Vector   dataVector;
    int      dataContribution;

    Node();
    Node( int nodenr );
    Node( int nodenr, double x, double y, double z );
    virtual ~Node();

    static void set_new_dof_limit( double value ){ eps_direct=value; }

    void set_nr( int nr );
    int get_nr() const;
    void add_dof( Dof_ptr plus_dof );
    Dof_ptr get_dof( int idof ) const;
    void get_dof( Dof::Type phys_type, Direction dir,
                  dof_sets *SystDofs ) const;
    void set_location( Location loc );
    void set_location( double x, double y, double z );
    Location get_location() const;
    void get_location( double& x, double& y, double& z ) const;
    void get_current_location( double& x, double& y, double& z ) const;
    void draw_location( double& x, double& y, double& z, double magnif ) const;
    int create_dof( Dof::dof_fixed fix_type, Dof::Type phys_type,
                    const Direction& dir );
    void make_global_dofs( int& nr_free, int& nr_fixed, int& nr_prescribed );
    void add_to_rhs( Dof::Load type, Direction dir, double magn,
                     SystemVector& rhs ) const;
    void add_to_prescribed( Dof::Type type, Direction dir,
                            double magn ) const;

    CartVector get_total_displacement() const;
    double get_temperature() const;

    virtual void WriteRestartData( std::ostream &restart ) const;
    virtual void ReadRestartData( std::istream &restart);
};

class IncNode:public Node
{
  public:
    IncNode():Node(){}
    IncNode( int nodenr ):Node( nodenr ){}
    IncNode( int nodenr, double x, double y, double z ):Node( nodenr, x, y, z ){}
    ~IncNode(){}

    void add_increments_to_totals();
    void reset_increments();
    void add_to_increments( const Vector &free,
                            const Vector &prescribed );
    void add_to_increments( const Vector &free );
    void get_incremental_solution_vector( Vector &free ) const;

    CartVector get_incremental_displacement() const;
    double get_temperature_increment() const;
};

} // namespace FeaTure

#endif /* NODES_H */
