// Copyright (c) 1999, A.H. van den Boogaard

#ifndef SHAPE_H
#define SHAPE_H

#include "ftmatrix.h"
#include "intpoint.h"

namespace FeaTure
{

class Shape
{
   protected:
      Shape(){}

   public:
      virtual ~Shape(){}
      virtual Matrix get_N_vec( const Vector &natcor ) const=0;
      virtual Matrix getPartialDerivatives( const Vector &natcor ) const=0;
      virtual IntLocs get_int_locs() const=0;
      virtual int get_nr_int_points() const=0;

      virtual IntLocs get_int_locs_face( int face_nr ) const=0;
      virtual Matrix dVdA( int face_nr ) const=0;
};

class Triangular:public Shape
{
   private:
      int int_scheme;

   protected:
      Triangular( int intscheme ):int_scheme(intscheme){}

   public:
      ~Triangular(){}
      IntLocs get_int_locs() const;
      int get_nr_int_points() const { return int_scheme; }

      Matrix dVdA( int face_nr ) const;
      IntLocs get_int_locs_face( int face_nr ) const;
};

class Tri_lin:public Triangular
{
   public:
      Tri_lin( int intscheme ):Triangular( intscheme ){}
      ~Tri_lin(){}
      Matrix get_N_vec( const Vector &natcor ) const;
      Matrix getPartialDerivatives( const Vector &natcor ) const;
};

class Tri_quad:public Triangular
{
   public:
      Tri_quad( int intscheme ):Triangular( intscheme ){}
      ~Tri_quad(){}
      Matrix get_N_vec( const Vector &natcor ) const;
      Matrix getPartialDerivatives( const Vector &natcor ) const;
};

class Tri_quad_alt:public Triangular
{
   public:
      Tri_quad_alt( int intscheme ):Triangular( intscheme ){}
      ~Tri_quad_alt(){}
      Matrix get_N_vec( const Vector &natcor ) const;
      Matrix getPartialDerivatives( const Vector &natcor ) const;
};

class Quadrilateral:public Shape
{
   private:
      int int_scheme[2];

   protected:
      Quadrilateral(int nxi, int neta){ int_scheme[0]=nxi; int_scheme[1]=neta; }

   public:
      ~Quadrilateral(){}
      IntLocs get_int_locs() const;
      int get_nr_int_points() const { return int_scheme[0]*int_scheme[1]; }

      Matrix dVdA( int face_nr ) const;
      IntLocs get_int_locs_face( int face_nr ) const;
};

class Quad_lin:public Quadrilateral
{
   public:
      Quad_lin( int nxi, int neta ):Quadrilateral( nxi, neta ){}
      ~Quad_lin(){}
      Matrix get_N_vec( const Vector &natcor ) const;
      Matrix getPartialDerivatives( const Vector &natcor ) const;
};

class Quad_quad:public Quadrilateral
{
   public:
      Quad_quad( int nxi, int neta ):Quadrilateral( nxi, neta ){}
      ~Quad_quad(){}
      Matrix get_N_vec( const Vector &natcor ) const;
      Matrix getPartialDerivatives( const Vector &natcor ) const;
};

class Quad_quad_alt:public Quadrilateral
{
   public:
      Quad_quad_alt( int nxi, int neta ):Quadrilateral( nxi, neta ){}
      ~Quad_quad_alt(){}
      Matrix get_N_vec( const Vector &natcor ) const;
      Matrix getPartialDerivatives( const Vector &natcor ) const;
};

class Hexagonal:public Shape
{
   private:
      int int_scheme[3];

   protected:
      Hexagonal( int nxi, int neta, int nzeta )
      { int_scheme[0]=nxi; int_scheme[1]=neta; int_scheme[2]=nzeta; }

   public:
      ~Hexagonal(){}
      IntLocs get_int_locs() const;
      int get_nr_int_points() const{ return int_scheme[0]*int_scheme[1]*int_scheme[2]; }

      Matrix dVdA( int face_nr ) const;
      IntLocs get_int_locs_face( int face_nr ) const;
};

class Hex_lin:public Hexagonal
{
   public:
      Hex_lin( int nxi, int neta, int nzeta ):Hexagonal( nxi, neta, nzeta ){}
      ~Hex_lin(){}
      Matrix get_N_vec( const Vector &natcor ) const;
      Matrix getPartialDerivatives( const Vector &natcor ) const;
};

} // namespace FeaTure

#endif // SHAPE_H
