// Copyright (c) 1999, A.H. van den Boogaard

#ifndef SYSMAT_H
#define SYSMAT_H

#include "ftmatrix.h"
#include "sprsmat.h"

namespace FeaTure
{

class SystemMat
{
   protected:
     SystemMat(){}

   public:
     virtual ~SystemMat(){}

     virtual void init(size_t row, size_t col)=0;
     virtual void clear()=0;
     virtual void add(size_t row, size_t col, double value)=0;
     virtual void print( char* name ) const=0;
     virtual void solve( const Vector& rhs, Vector& x )=0;
};

class SparseSysMat:public SystemMat, public CR_Matrix
{
    private:
     size_t    size;
     bool      symmetric;
     SystemMat *Sky;

    public:
     SparseSysMat():CR_Matrix(),size(0),symmetric(false),Sky(NULL){}
     SparseSysMat(size_t N, bool symm=false):CR_Matrix(N,N),size(N),
                                              symmetric(symm),Sky(NULL){}
     SparseSysMat(const SparseSysMat &S):SystemMat(),CR_Matrix(S),size(S.size),
                                              symmetric(S.symmetric),Sky(NULL){}
     ~SparseSysMat(){ delete Sky; }

     void init(size_t row, size_t col);
     void clear();
     inline void add(size_t row, size_t col, double value)
                    { set(row-1,col-1) += value; }
     void add(const SparseSysMat &A, double s=1.0 );
     void print( char* name ) const;
     void solve( const Vector& rhs, Vector& x );
};

class SkySymMat:public SystemMat
{
   private:
     size_t *p, size;
     bool   decomposed;
     double **a;

     void create_mem();
     int factor();
     void substitute( Vector &x, const Vector &y ) const;

   public:
     SkySymMat();
     SkySymMat( size_t size );
     SkySymMat( size_t size, size_t *p );
     SkySymMat( const SkySymMat &S );
     ~SkySymMat();

     void init(size_t row, size_t col);
     void clear();
     void add(size_t row, size_t col, double value);
     void print( char* name ) const;
     void solve( const Vector& rhs, Vector& x );
};

class DiagonalSysMat:public SystemMat
{
   private:
     size_t size;
     bool   inverted;
     double *a;

   public:
     DiagonalSysMat();
     DiagonalSysMat( size_t size );
     DiagonalSysMat( const DiagonalSysMat &D );
     ~DiagonalSysMat();

     void init(size_t row, size_t col);
     void clear();
     void add(size_t row, size_t col, double value);
     void print( char* name ) const;
     void solve( const Vector& rhs, Vector& x );
     void mult( const Vector& x, Vector* b ) const;
     void selfmult( double s );
};

void subspace_iteration( SparseSysMat &A, const SparseSysMat &B,
                         Vector *lambda, Matrix *V, bool init=true );

} // namespace FeaTure

#endif // SYSMAT_H
