// Copyright (c) 1999, A.H. van den Boogaard

#include "intpoint.h"
#include "ftexcept.h"
#include <sstream>

using FeaTure::Vector;
using FeaTure::program_error;
using FeaTure::IntLocs;

void FeaTure::Gauss( int ip, int nip, double& xi, double& weight )
{
   if ( ip<1 || ip>nip )
   {
      std::ostringstream message;
      message << "wrong integration point number:" << ip <<
                 " of " << nip << " available";
      throw program_error( message.str() );
   }

   switch ( nip )
   {
      case  1: xi = 0.0;
               weight = 2.0;
               break;
      case  2: switch ( ip )
               {
               case 1: xi = -0.577350269189626;
                       break;
               case 2: xi =  0.577350269189626;
                       break;
               }
               weight = 1.0;
               break;
      case  3: switch ( ip )
               {
               case 1: xi     = -0.774596669241483;
                       weight =  0.555556;
                       break;
               case 2: xi     =  0.0;
                       weight =  0.888888888888889;
                       break;
               case 3: xi     =  0.774596669241483;
                       weight =  0.555555555555556;
                       break;
               }
               break;
      case  4: switch ( ip )
               {
               case 1: xi     = -0.861136311594053;
                       weight =  0.347854845137454;
                       break;
               case 2: xi     = -0.339981043584856;
                       weight =  0.652145154862546;
                       break;
               case 3: xi     =  0.339981043584856;
                       weight =  0.652145154862546;
                       break;
               case 4: xi     =  0.861136311594053;
                       weight =  0.347854845137454;
                       break;
               }
               break;
      case  5: switch ( ip )
               {
               case 1: xi     = -0.906179845938664;
                       weight =  0.236926885056189;
                       break;
               case 2: xi     = -0.538469310105683;
                       weight =  0.478628670499366;
                       break;
               case 3: xi     =  0.0;
                       weight =  0.568888888888889;
                       break;
               case 4: xi     =  0.906179845938664;
                       weight =  0.236926885056189;
                       break;
               case 5: xi     =  0.538469310105683;
                       weight =  0.478628670499366;
                       break;
               }
               break;
      case  6: switch ( ip )
               {
               case 1: xi     = -0.932469514203152;
                       weight =  0.171324492379170;
                       break;
               case 2: xi     = -0.661209386466265;
                       weight =  0.360761573048139;
                       break;
               case 3: xi     = -0.238619186083197;
                       weight =  0.467913934572691;
                       break;
               case 4: xi     =  0.932469514203152;
                       weight =  0.171324492379170;
                       break;
               case 5: xi     =  0.661209386466265;
                       weight =  0.360761573048139;
                       break;
               case 6: xi     =  0.238619186083197;
                       weight =  0.467913934572691;
                       break;
               }
               break;
      default: { std::ostringstream message;
                 message << "wrong nr of integration points: " << nip;
                 throw program_error( message.str() );
               }
   }
}

void FeaTure::Hammer( int ip, int nip, double& L1, double& L2, double& L3,
                      double& weight )
{
   if ( ip<1 || ip>nip )
   {
      std::ostringstream message;
      message << "wrong integration point number:" << ip <<
                 " of " << nip << " available";
      throw program_error( message.str() );
   }

   switch ( nip )
   {
      case  1: L1 = L2 = L3 = 0.333333333333333;
               weight = 0.5;
               break;
    /*  case  3: switch ( ip )   // alternative scheme degree of precision = 2
                                 // unsuitable for axisymmetric, division by r=0
               {
               case 1: L1 = 0.5; L2 = 0.5; L3 = 0.0;
                       break;
               case 2: L1 = 0.5; L2 = 0.0; L3 = 0.5;
                       break;
               case 3: L1 = 0.0; L2 = 0.5; L3 = 0.5;
                       break;
               }
               weight = 0.166666666666667;   // 1.0/6.0;
               break; */
       case  3: switch ( ip )    // alternative scheme  degree of precision = 2
               {
               case 1: L1 = 0.166666666666667; L2 = 0.166666666666667; L3 = 0.666666666666667;
                       break;
               case 2: L1 = 0.666666666666667; L2 = 0.166666666666667; L3 = 0.166666666666667;
                       break;
               case 3: L1 = 0.166666666666667; L2 = 0.666666666666667; L3 = 0.166666666666667;
                       break;
               }
               weight = 0.166666666666667;   // 1.0/6.0;
               break;
      case  4: switch ( ip )
               {
               case 1: L1 = L2 = L3 = 0.333333333333333;  // 1.0/3.0;
                       weight =  -0.28125;   // -9.0/32.0;
                       break;
               case 2: L1 = 0.6; L2 = L3 = 0.2;
                       weight =  0.260416666666667;   // 25.0/96.0;
                       break;
               case 3: L2 = 0.6; L1 = L3 = 0.2;
                       weight =  0.260416666666667;   // 25.0/96.0;
                       break;
               case 4: L1 = L2 = 0.2; L3 = 0.6;
                       weight =  0.260416666666667;   //25.0/96.0;
                       break;
               }
               break;
      case  7: switch ( ip )   // degree of precision = 5
               {
               case 1: L1 = L2 = 0.1012865073235; L3 = 0.7974269853531;
                       weight = 0.0629695902724;
                       break;
               case 2: L1 = 0.7974269853531; L2 = L3 = 0.1012865073235;
                       weight = 0.0629695902724;
                       break;
               case 3: L1 = L3 = 0.1012865073235; L2 = 0.7974269853531;
                       weight = 0.0629695902724;
                       break;
               case 4: L1 = L3 = 0.4701420641051; L2 = 0.0597158717898;
                       weight =  0.0661970763943;
                       break;
               case 5: L1 = L2 = 0.4701420641051; L3 = 0.0597158717898;
                       weight =  0.0661970763943;
                       break;
               case 6: L2 = L3 = 0.4701420641051; L1 = 0.0597158717898;
                       weight =  0.0661970763943;
                       break;
               case 7: L1 = L2 = L3 = 0.3333333333333;
                       weight = 0.1125;
                       break;
               }
               break;
      default: { std::ostringstream message;
                 message << "wrong nr of integration points: " << nip;
                 throw program_error( message.str() );
               }
   }
}

IntLocs::IntLocs()
{
   nr_of_intpt = 0;
   intpts = NULL;
}

IntLocs::IntLocs( int nr )
{
   nr_of_intpt = nr;
   intpts = new int_loc[nr];
}

IntLocs::IntLocs( const IntLocs& intlocs )
{
   nr_of_intpt = intlocs.nr_of_intpt;
   if ( nr_of_intpt == 0 )
   {
      intpts = NULL;
      return;
   }
   intpts = new int_loc[nr_of_intpt];
   for ( int ip=0; ip<nr_of_intpt; ip++ )
   {
      intpts[ip] = intlocs.intpts[ip];
   }
}

IntLocs::~IntLocs()
{
   if ( intpts != NULL ) delete [] intpts;
}

void IntLocs::set_nr_intpt( int nr )
{
   if ( intpts != NULL ) delete [] intpts;
   nr_of_intpt = nr;
   intpts = new int_loc[nr];
}

int IntLocs::get_nr_intpt() const
{
   return nr_of_intpt;
}

void IntLocs::set_values( int ip, Vector coor, double wght )
{
   if ( ip < 1 || ip > nr_of_intpt )
   {
      std::ostringstream message;
      message << "wrong integration point number: " << ip << std::endl;
      message << "should be between 1 and " << nr_of_intpt;
      throw program_error( message.str() );
   }

   intpts[ip-1].nat_coord = coor;
   intpts[ip-1].weight = wght;
}

Vector IntLocs::get_coor(int ip) const
{
   return ( intpts[ip-1].nat_coord );
}

double IntLocs::get_weight(int ip) const
{
   return ( intpts[ip-1].weight );
}
