// Copyright (c) 2001, A.H. van den Boogaard

#include "sprsmat.h"

using FeaTure::SparseVector;
using FeaTure::Vector;
using FeaTure::Matrix;
using FeaTure::CR_Matrix;
using FeaTure::Subscript;


void SparseVector::set(Subscript i, const double& t)
{
    iterator p = std::lower_bound( c_.begin(), c_.end(), i );

    if (p == c_.end() || p->index()!=i ) // if not found...
    {
        c_.insert( p, ai(i,t) );
        if (i >= Dim_) Dim_ = i+1;      // keep track of largest index
    }
    else
        p->value() = t;
}

double& SparseVector::set(Subscript i)
{
    iterator p = std::lower_bound( c_.begin(), c_.end(), i );

    if ( p == c_.end() || p->index()!=i ) // if not found...
    {
        iterator n = c_.insert( p, ai(i,double(0)) );
        if ( i > Dim_) Dim_ = i+1;      // keep track of largest index
        return n->value();
    }
    else
        return p->value();
}

const double& SparseVector::get(Subscript i) const
{
    static const double zero(0);
    const_iterator p = std::lower_bound( c_.begin(), c_.end(), i );
    if (p == c_.end() || p->index()!=i )
        return zero;
    else
        return p->value() ;
}

namespace {
// an unnamed (=local) namespace

inline double spvec_dot_product( const SparseVector &A,
                                 const Vector &B,
                                 const double &prev_sum)
{
    assert( A.dim() <= B.size() );

    SparseVector::const_iterator p;
    SparseVector::const_iterator p_end=A.end();
 
    double sum = prev_sum;
 
    for ( p=A.begin(); p!=p_end; p++ )
        sum += p->value() * B[p->index()];
 
    return sum;
}

} // end of local namespace

// y <-- A * x       CR_Matrix  *  Vector

void FeaTure::cr_mat_vec_mult(const CR_Matrix &A,
                              const Vector &x, Vector *Yptr)
{
    Vector &Y = *Yptr;
    Y.resize(A.num_rows());
    double  t(0);

    CR_Matrix::const_iterator p;
    CR_Matrix::const_iterator p_end = A.end();

    int i(0);
    for (p=A.begin(); p!=p_end; p++, i++)
       Y[i] = spvec_dot_product( *p, x, t);
}

// Y <-- A * X       CR_Matrix  *  Matrix

void FeaTure::cr_mat_mat_mult(const CR_Matrix &A,
                              const Matrix &X, Matrix *Yptr)
{
    Matrix &Y = *Yptr;
    size_t n( X.size1() );
    size_t m( X.size2() );
    Y.resize(A.num_rows(),m);
    double  t(0);

    CR_Matrix::const_iterator p;
    CR_Matrix::const_iterator p_end = A.end();

    for ( size_t j=0; j<m; j++ )
    {
       Vector x(n);
       for ( size_t i=0; i<n; i++ ) x[i] = X[i][j];

       int i(0);
       for (p=A.begin(); p!=p_end; p++, i++)
          Y[i][j] = spvec_dot_product( *p, x, t);
    }
}
